function [p, wx, wx_pic, h1, h2] = Wavelet(t, x, pmin, pmax, sat)
% WAVELET computes wavelet transform and plot it.
%#########################################
%#                                       #
%#    [Function]                         #
%#    Wavelet (WL) Transform and Plot    #
%# origin code adopted from Dr. Ben Chao #
%#########################################
%
% This program computes the 1-D wavelet transform of a REAL time series.
%  (The result reflects the positive/negative phase
%  undulations of the time series. The transformation is actually
% performed using a convolution which, in turn, uses FFT.)
% Note: Only works for equally spaced time series. For un-equally spaced time series, you must
% interpolate/decimate first (don't forget to low-pass filter before you decimate.)
%
%          format: [p,wx]=wavelet(x)
% Output wx is the cosine (symmetric, real) part of
%  the wavelet spectrum, containing about 100 "levels" for plotting.
%
% >> [p,wx] = wavelet(t,x,pmin,pmax)                   <- for non-plotting
% >> [p,wx] = wavelet(t,x,pmin,pmax,sat)               <- for plotting
% >> [p,wx,wx_pic,h1,h2] = wavelet(t,x,pmin,pmax,sat)  <- output plotting data and image handles
%
%----Input Variables----------------------------------------------------------
% t       -> Time Sequence, time array. 1-D vector.
% x       -> Time Sequence data, x=f(t). Must be Equivalent Time Sampling. 1-D vector.
% pmin    -> minimum display period of the WL transform. Unit must be the same with t.
% pmax    -> maximum display period of the WL transform. Unit must be the same with t.
% sat     -> color saturation level (in %) for plot. 
%----Output Variables---------------------------------------------------------
% p              -> Period spacing (log2 scale) sequence, at least ~100 points
% wx             -> Transformed x (time-period array) 
% wx_pic         -> Transformed x, scaled for plotting, under given saturation option
% h1             -> Handle of original wave plot
% h2             -> Handle of WL transform plot
%============================================================

dt=t(2)-t(1);
%----Only accept when input t and x are vectors----

if numel(t) ~= length(t) | numel(x) ~= length(x)
  error('data format error!')
else
  n = length(x);
  x = reshape(detrend(x), 1, n);
end

%----0Set parameters----

mu = pi * sqrt(2 / log(2));
np = ceil(log2(pmax / pmin)) * 4 + 1;
fact = ceil(100 / np);
if fact > 1
   np = np * fact - fact + 1;
end
p = logspace(log10(pmin), log10(pmax), np);

%----construct the wavelet and convolve by FFT...----

a = mu * p / (2 * pi);
wx = zeros(np,n);
for m = 1 : np
   nw2 = ceil(5 * p(m) / dt);
   nw = 2 * nw2 + 1;
   tw = linspace(-nw2 * dt, nw2 * dt, nw);
   w = exp(-tw .^ 2 / (2 * a(m) ^ 2)) .* cos(mu * tw / a(m)) / sqrt(2 * pi * a(m));

%====(WJ: it's the previous message)=========================

% The (unused) set of codes below, using fft for convolution (due to Cheh Pan),
% yields the same result as convolution (even with
% padded zeros.... this I do NOT understand....)
%   num=n+nw-1;
%   nn=2^(nextpow2(num));
%  y=real(ifft(fft(x,nn).*fft(w,nn)));

%============================================================

% The convolution works below because the wavelet is an even function
% and hence can be flipped in time.
  y = conv(w, x);
  wx(m, :) = y(nw2 + 1 : nw2 + n) * dt / sqrt(a(m));
end

%----Plot----

if nargin == 5
  subplot(3, 1, 1)
    h1 = plot(t, x,'linewidth',1.5);
    axis tight;
    set(gca, 'XTickLabel', datestr(get(gca, 'XTick'), 'yyyy/mm'));
    xlabel('time');
  subplot(3, 1, 2:3)
    %----Set color saturation above/below 'sat'----

    val_up_satur = max(max(wx)) * sat / 100;
    val_dn_satur = min(min(wx)) * sat / 100;
    wx_pic = wx;
    wx_pic(wx_pic > val_up_satur) = val_up_satur;
    wx_pic(wx_pic < val_dn_satur) = val_dn_satur;
    wx_pic = fix(63 * (wx_pic - val_dn_satur) / (val_up_satur - val_dn_satur) + 1);

    %----Draw----
 
    h2 = image(t, log2(p), wx_pic);
    axis tight;
%    new_YTickLabel = 2 .^ str2num(get(gca, 'YTickLabel'));
    new_YTickLabel = 2 .^ get(gca, 'YTick');
    set(gca, 'YTickLabel', new_YTickLabel);
    set(gca, 'YTickMode', 'manual');     % Prevents resampling ticks when resizing the figure
    set(gca, 'XTickLabel', datestr(get(gca,'XTick'),'yyyy/mm'));
    xlabel('time'); ylabel('period (in days)');
    title('wavelet spectrogram');
    colormap(jet);
    colorbar('southoutside');
    caxis([0 80]);
end

